@@ -1,15 +1,16 @@
+# validated: 2017-10-03 EN e1195e8b9dab edu/wpi/first/wpilibj/command/PrintCommand.java
 #----------------------------------------------------------------------------
-# Copyright (c) FIRST 2008-2012. All Rights Reserved.
+# Copyright (c) FIRST 2008-2016. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
 #----------------------------------------------------------------------------
 
-from .command import Command
+from .instantcommand import InstantCommand
 
 __all__ = ["PrintCommand"]
 
-class PrintCommand(Command):
+class PrintCommand(InstantCommand):
     """A PrintCommand is a command which prints out a string when it is
     initialized, and then immediately finishes.
 
@@ -28,6 +29,3 @@ class PrintCommand(Command):
 
     def initialize(self):
         print(self.message)
-
-    def isFinished(self):
-        return True
