@@ -1,3 +1,4 @@
+# validated: 2017-12-15 EN f9bece2ffbf7 edu/wpi/first/wpilibj/Counter.java
 #----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -6,24 +7,30 @@
 #----------------------------------------------------------------------------
 
 import hal
-import warnings
 import weakref
+from enum import IntEnum
 
 from .interfaces.counterbase import CounterBase
 from .interfaces.pidsource import PIDSource
 from .analogtriggeroutput import AnalogTriggerOutput
 from .digitalinput import DigitalInput
-from .livewindow import LiveWindow
 from .sensorbase import SensorBase
 from ._impl.utils import match_arglist, HasAttribute
 
 __all__ = ["Counter"]
 
-def _freeCounter(counter):
-    hal.setCounterUpdateWhenEmpty(counter, True)
-    hal.clearCounterUpSource(counter)
-    hal.clearCounterDownSource(counter)
-    hal.freeCounter(counter)
+def _freeCounter(counterObj):
+    hal.setCounterUpdateWhenEmpty(counterObj._counter, True)
+    
+    counterObj.clearUpSource()
+    counterObj.clearDownSource()
+
+    hal.freeCounter(counterObj._counter)
+
+    counterObj.upSource = None
+    counterObj.downSource = None
+    counterObj._counter = None
+
 
 class Counter(SensorBase):
     """Counts the number of ticks on a :class:`.DigitalInput` channel.
@@ -38,7 +45,7 @@ class Counter(SensorBase):
     .. not_implemented: initCounter
     """
 
-    class Mode:
+    class Mode(IntEnum):
         """Mode determines how and what the counter counts"""
         
         #: two pulse mode
@@ -54,7 +61,7 @@ class Counter(SensorBase):
         kExternalDirection = 3
 
     EncodingType = CounterBase.EncodingType
-    PIDSourceParameter = PIDSource.PIDSourceParameter
+    PIDSourceType = PIDSource.PIDSourceType
     allocatedUpSource = False
     allocatedDownSource = False
 
@@ -71,11 +78,10 @@ class Counter(SensorBase):
         - (none)
         - upSource
         - upSource, down source
-        And, to keep consistency with Java wpilib.
         - encodingType, up source, down source, inverted
 
         If the passed object has a
-        `getChannelForRouting` function, it is assumed to be a DigitalSource.
+        `getPortHandleForRouting` function, it is assumed to be a DigitalSource.
         If the passed object has a `createOutput` function, it is assumed to
         be an AnalogTrigger.
 
@@ -97,9 +103,11 @@ class Counter(SensorBase):
             Either k1X or k2X to indicate 1X or 2X decoding. 4X decoding
             is not supported by Counter; use `Encoder` instead.  Defaults
             to k1X if unspecified.  Only used when two sources are specified.
+        :type encodingType: :class:`.Counter.EncodingType`
         """
 
-        source_identifier = [int, HasAttribute("getChannelForRouting"), HasAttribute("createOutput")]
+        super().__init__()
+        source_identifier = [int, HasAttribute("getPortHandleForRouting"), HasAttribute("createOutput")]
 
         argument_templates = [[],
                               [("upSource", source_identifier), ],
@@ -128,24 +136,28 @@ class Counter(SensorBase):
 
         # save some variables
         self.distancePerPulse = 1.0 # distance of travel for each tick
-        self.pidSource = PIDSource.PIDSourceParameter.kDistance
+        self.pidSource = self.PIDSourceType.kDisplacement
 
         # create counter
         self._counter, self.index = hal.initializeCounter(mode)
-        self._counter_finalizer = \
-            weakref.finalize(self, _freeCounter, self._counter)
+        self.__finalizer = \
+            weakref.finalize(self, _freeCounter, self)
 
         self.setMaxPeriod(.5)
 
-        hal.HALReport(hal.HALUsageReporting.kResourceType_Counter, self.index,
-                      mode)
+        hal.report(hal.UsageReporting.kResourceType_Counter, self.index, mode)
+        self.setName("Counter", self.index)
 
-        #Set sources
+        # Set sources
         if upSource is not None:
             self.setUpSource(upSource)
+        else:
+            self.upSource = None
 
         if downSource is not None:
             self.setDownSource(downSource)
+        else:
+            self.downSource = None
 
         # when given two sources, set edges
         if upSource is not None and downSource is not None:
@@ -159,16 +171,13 @@ class Counter(SensorBase):
 
     @property
     def counter(self):
-        if not self._counter_finalizer.alive:
-            return None
+        if not self.__finalizer.alive:
+            raise ValueError("Cannot use counter after free() has been called")
         return self._counter
 
     def free(self):
-        self.setUpdateWhenEmpty(True)
-        self.clearUpSource()
-
-        self.clearDownSource()
-        self._counter_finalizer()
+        super().free()
+        self.__finalizer()
 
     def getFPGAIndex(self):
         """
@@ -188,7 +197,7 @@ class Counter(SensorBase):
         - analogTrigger, triggerType
 
         For positional arguments, if the passed object has a
-        `getChannelForRouting` function, it is assumed to be a DigitalSource.
+        `getPortHandleForRouting` function, it is assumed to be a DigitalSource.
         If the passed object has a `createOutput` function, it is assumed to
         be an AnalogTrigger.
 
@@ -207,16 +216,13 @@ class Counter(SensorBase):
         :param triggerType:
             The analog trigger output that will trigger the counter.
             Defaults to kState if not specified.
-        :type triggerType: AnalogTriggerType
+        :type triggerType: AnalogTrigger.AnalogTriggerType
         """
 
         #TODO Both this and the java implementation should probably not allow setting a source if one is already set.
 
-        if self.counter is None:
-            raise ValueError("operation on freed port")
-
         argument_templates = [[("channel", int)],
-                              [("source", HasAttribute("getChannelForRouting")), ],
+                              [("source", HasAttribute("getPortHandleForRouting")), ],
                               [("analogTrigger", HasAttribute("createOutput"))],
                               [("analogTrigger", HasAttribute("createOutput")), ("triggerType", None)]]
 
@@ -234,6 +240,7 @@ class Counter(SensorBase):
             if channel is not None:
                 source = DigitalInput(channel)
                 self.allocatedUpSource = True
+                self.addChild(source)
             elif analogTrigger is not None and triggerType is not None:
                 source = analogTrigger.createOutput(triggerType)
             else:
@@ -242,8 +249,8 @@ class Counter(SensorBase):
         # save and set
         self.upSource = source
         hal.setCounterUpSource(self.counter,
-                               self.upSource.getChannelForRouting(),
-                               self.upSource.getAnalogTriggerForRouting())
+                               self.upSource.getPortHandleForRouting(),
+                               self.upSource.getAnalogTriggerTypeForRouting())
 
     def setUpSourceEdge(self, risingEdge, fallingEdge):
         """Set the edge sensitivity on an up counting source. Set the up
@@ -256,8 +263,6 @@ class Counter(SensorBase):
         """
         if self.upSource is None:
             raise ValueError("Up Source must be set before setting the edge")
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterUpSourceEdge(self.counter, risingEdge, fallingEdge)
 
     def clearUpSource(self):
@@ -266,10 +271,7 @@ class Counter(SensorBase):
             self.upSource.free()
             self.allocatedUpSource = False
         self.upSource = None
-
-        if self.counter is None:
-            return
-        hal.clearCounterUpSource(self.counter)
+        hal.clearCounterUpSource(self._counter)
 
     def setDownSource(self, *args, **kwargs):
         """Set the down counting source for the counter.
@@ -302,20 +304,17 @@ class Counter(SensorBase):
         :param triggerType:
             The analog trigger output that will trigger the counter.
             Defaults to kState if not specified.
-        :type triggerType: AnalogTriggerType
+        :type triggerType: AnalogTrigger.AnalogTriggerType
         """
 
         #TODO Both this and the java implementation should probably not allow setting a source if one is already set.
 
-        if self.counter is None:
-            raise ValueError("operation on freed port")
-
         argument_templates = [[("channel", int)],
-                              [("source", HasAttribute("getChannelForRouting")), ],
+                              [("source", HasAttribute("getPortHandleForRouting")), ],
                               [("analogTrigger", HasAttribute("createOutput")), ],
                               [("analogTrigger", HasAttribute("createOutput")), ("triggerType", None)]]
 
-        _, results = match_arglist('Counter.setUpSource',
+        _, results = match_arglist('Counter.setDownSource',
                                    args, kwargs, argument_templates)
 
         # extract arguments
@@ -329,6 +328,7 @@ class Counter(SensorBase):
             if channel is not None:
                 source = DigitalInput(channel)
                 self.allocatedDownSource = True
+                self.addChild(source)
             elif analogTrigger is not None and triggerType is not None:
                 source = analogTrigger.createOutput(triggerType)
             else:
@@ -337,8 +337,8 @@ class Counter(SensorBase):
         # save and set
         self.downSource = source
         hal.setCounterDownSource(self.counter,
-                                 self.downSource.getChannelForRouting(),
-                                 self.downSource.getAnalogTriggerForRouting())
+                                 self.downSource.getPortHandleForRouting(),
+                                 self.downSource.getAnalogTriggerTypeForRouting())
 
     def setDownSourceEdge(self, risingEdge, fallingEdge):
         """Set the edge sensitivity on an down counting source. Set the down
@@ -351,8 +351,7 @@ class Counter(SensorBase):
         """
         if self.downSource is None:
             raise ValueError("Down Source must be set before setting the edge")
-        if self.counter is None:
-            raise ValueError("operation on freed port")
+
         hal.setCounterDownSourceEdge(self.counter, risingEdge, fallingEdge)
 
     def clearDownSource(self):
@@ -363,16 +362,12 @@ class Counter(SensorBase):
             self.allocatedDownSource = False
         self.downSource = None
 
-        if self.counter is None:
-            return
-        hal.clearCounterDownSource(self.counter)
+        hal.clearCounterDownSource(self._counter)
 
     def setUpDownCounterMode(self):
         """Set standard up / down counting mode on this counter. Up and down
         counts are sourced independently from two inputs.
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterUpDownMode(self.counter)
 
     def setExternalDirectionMode(self):
@@ -380,8 +375,6 @@ class Counter(SensorBase):
         the Up counter input. The Down counter input represents the direction
         to count.
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterExternalDirectionMode(self.counter)
 
     def setSemiPeriodMode(self, highSemiPeriod):
@@ -391,8 +384,6 @@ class Counter(SensorBase):
         :param highSemiPeriod: True to count up on both rising and falling
         :type  highSemiPeriod: bool
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterSemiPeriodMode(self.counter, highSemiPeriod)
 
     def setPulseLengthMode(self, threshold):
@@ -404,8 +395,6 @@ class Counter(SensorBase):
             opposite direction. Units are seconds.
         :type  threshold: float, int
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterPulseLengthMode(self.counter, float(threshold))
 
     def get(self):
@@ -413,8 +402,6 @@ class Counter(SensorBase):
         may still be running, so it reflects the current value. Next time it
         is read, it might have a different value.
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         return hal.getCounter(self.counter)
 
     def getDistance(self):
@@ -431,8 +418,6 @@ class Counter(SensorBase):
         doesn't effect the running state of the counter, just sets the
         current value to zero.
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.resetCounter(self.counter)
 
     def setMaxPeriod(self, maxPeriod):
@@ -445,8 +430,6 @@ class Counter(SensorBase):
             considered moving in seconds.
         :type maxPeriod: float or int
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterMaxPeriod(self.counter, float(maxPeriod))
 
     def setUpdateWhenEmpty(self, enabled):
@@ -467,8 +450,6 @@ class Counter(SensorBase):
         :param enabled: True to continue updating
         :type  enabled: bool
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterUpdateWhenEmpty(self.counter, enabled)
 
     def getStopped(self):
@@ -482,8 +463,6 @@ class Counter(SensorBase):
             MaxPeriod value set by SetMaxPeriod.
         :rtype: bool
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         return hal.getCounterStopped(self.counter)
 
     def getDirection(self):
@@ -492,8 +471,6 @@ class Counter(SensorBase):
         :returns: The last direction the counter value changed.
         :rtype: bool
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         return hal.getCounterDirection(self.counter)
 
     def setReverseDirection(self, reverseDirection):
@@ -505,8 +482,6 @@ class Counter(SensorBase):
         :param reverseDirection: True if the value counted should be negated.
         :type  reverseDirection: bool 
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterReverseDirection(self.counter, reverseDirection)
 
     def getPeriod(self):
@@ -517,8 +492,6 @@ class Counter(SensorBase):
         :returns: The period of the last two pulses in units of seconds.
         :rtype: float
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         return hal.getCounterPeriod(self.counter)
 
     def getRate(self):
@@ -540,8 +513,6 @@ class Counter(SensorBase):
         :param samplesToAverage: The number of samples to average from 1 to 127.
         :type  samplesToAverage: int
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         hal.setCounterSamplesToAverage(self.counter, samplesToAverage)
 
     def getSamplesToAverage(self):
@@ -553,8 +524,6 @@ class Counter(SensorBase):
         :returns: The number of samples being averaged (from 1 to 127)
         :rtype: int
         """
-        if self.counter is None:
-            raise ValueError("operation on freed port")
         return hal.getCounterSamplesToAverage(self.counter)
 
     def setDistancePerPulse(self, distancePerPulse):
@@ -571,39 +540,30 @@ class Counter(SensorBase):
         """
         self.distancePerPulse = distancePerPulse
 
-    def setPIDSourceParameter(self, pidSource):
+    def setPIDSourceType(self, pidSource):
         """Set which parameter of the encoder you are using as a process
         control variable. The counter class supports the rate and distance
         parameters.
 
         :param pidSource: An enum to select the parameter.
-        :type  pidSource: :class:`Counter.PIDSourceParameter`
+        :type  pidSource: :class:`Counter.PIDSourceType`
         """
-        if pidSource not in (self.PIDSourceParameter.kDistance,
-                             self.PIDSourceParameter.kRate):
+        if pidSource not in (self.PIDSourceType.kDisplacement,
+                             self.PIDSourceType.kRate):
             raise ValueError("Invalid pidSource argument '%s'" % pidSource)
         self.pidSource = pidSource
 
+    def getPIDSourceType(self):
+        return self.pidSource
+
     def pidGet(self):
-        if self.pidSource == self.PIDSourceParameter.kDistance:
+        if self.pidSource == self.PIDSourceType.kDisplacement:
             return self.getDistance()
-        elif self.pidSource == self.PIDSourceParameter.kRate:
+        elif self.pidSource == self.PIDSourceType.kRate:
             return self.getRate()
         else:
             return 0.0
 
-    # Live Window code, only does anything if live window is activated.
-
-    def getSmartDashboardType(self):
-        return "Counter"
-
-    def updateTable(self):
-        table = self.getTable()
-        if table is not None:
-            table.putNumber("Value", self.get())
-
-    def startLiveWindowMode(self):
-        pass
-
-    def stopLiveWindowMode(self):
-        pass
+    def initSendable(self, builder):
+        builder.setSmartDashboardType("Counter")
+        builder.addDoubleProperty("Value", self.get, None)
