@@ -1,3 +1,4 @@
+# validated: 2018-01-01 EN c647a801ad80 edu/wpi/first/wpilibj/Encoder.java
 #----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -8,6 +9,7 @@
 import hal
 import warnings
 import weakref
+import enum
 
 from .interfaces.counterbase import CounterBase
 from .interfaces.pidsource import PIDSource
@@ -25,17 +27,16 @@ def _freeEncoder(encoder):
     hal.freeEncoder(encoder)
 
 class Encoder(SensorBase):
-    """Reads from quadrature encoders.
+    """Class to read quadrature encoders.
     
     Quadrature encoders are devices that count
-    shaft rotation and can sense direction. The output of the QuadEncoder class
+    shaft rotation and can sense direction. The output of the Encoder class
     is an integer that can count either up or down, and can go negative for
-    reverse direction counting. When creating QuadEncoders, a direction is
-    supplied that changes the sense of the output to make code more readable
+    reverse direction counting. When creating Encoders, a direction can be
+    supplied that inverts the sense of the output to make code more readable
     if the encoder is mounted such that forward movement generates negative
     values. Quadrature encoders have two digital outputs, an A Channel and a
-    B Channel that are out of phase with each other to allow the FPGA to do
-    direction sensing.
+    B Channel, that are out of phase with each other for direction sensing.
 
     All encoders will immediately start counting - reset() them if you need
     them to be zeroed before use.
@@ -45,18 +46,16 @@ class Encoder(SensorBase):
     - aSource: The A phase of the quad encoder
     - bSource: The B phase of the quad encoder
     - indexSource: The index source (available on some encoders)
-    
-    .. not_implemented: initEncoder
     """
 
-    class IndexingType:
+    class IndexingType(enum.IntEnum):
         kResetWhileHigh = 0
         kResetWhileLow = 1
         kResetOnFallingEdge = 2
         kResetOnRisingEdge = 3
 
     EncodingType = CounterBase.EncodingType
-    PIDSourceParameter = PIDSource.PIDSourceParameter
+    PIDSourceType = PIDSource.PIDSourceType
 
     def __init__(self, *args, **kwargs):
         """Encoder constructor. Construct a Encoder given a and b channels
@@ -65,7 +64,7 @@ class Encoder(SensorBase):
         The encoder will start counting immediately.
 
         The a, b, and optional index channel arguments may be either channel
-        numbers or `DigitalSource` sources. There may also be a boolean
+        numbers or :class:`.DigitalSource` sources. There may also be a boolean
         reverseDirection, and an encodingType according to the following
         list.
         
@@ -81,7 +80,7 @@ class Encoder(SensorBase):
         - aChannel, bChannel, indexChannel
 
         For positional arguments, if the passed object has a
-        `getChannelForRouting` function, it is assumed to be a DigitalSource.
+        `getPortHandleForRouting` function, it is assumed to be a DigitalSource.
 
         Alternatively, sources and/or channels may be passed as keyword
         arguments.  The behavior of specifying both a source and a number
@@ -92,19 +91,26 @@ class Encoder(SensorBase):
         and inputType.
 
         :param aSource: The source that should be used for the a channel.
+        :type aSource: :class:`.DigitalSource`
         :param bSource: The source that should be used for the b channel.
+        :type bSource: :class:`.DigitalSource`
         :param indexSource: The source that should be used for the index
             channel.
+        :type indexSource: :class:`.DigitalSource`
         :param aChannel: The digital input index that should be used for
             the a channel.
+        :type aChannel: int
         :param bChannel: The digital input index that should be used for
             the b channel.
+        :type bChannel: int
         :param indexChannel: The digital input index that should be used
             for the index channel.
+        :type indexChannel: int
         :param reverseDirection:
             Represents the orientation of the encoder and inverts the
             output values if necessary so forward represents positive
             values.  Defaults to False if unspecified.
+        :type reverseDirection: bool
         :param encodingType:
             Either k1X, k2X, or k4X to indicate 1X, 2X or 4X decoding. If
             4X is selected, then an encoder FPGA object is used and the
@@ -115,9 +121,10 @@ class Encoder(SensorBase):
             spec'd count.  Defaults to k4X if unspecified.
         :type encodingType: :class:`Encoder.EncodingType`
         """
-        a_source_arg = ("aSource", HasAttribute("getChannelForRouting"))
-        b_source_arg = ("bSource", HasAttribute("getChannelForRouting"))
-        index_source_arg = ("indexSource", HasAttribute("getChannelForRouting"))
+        super().__init__()
+        a_source_arg = ("aSource", HasAttribute("getPortHandleForRouting"))
+        b_source_arg = ("bSource", HasAttribute("getPortHandleForRouting"))
+        index_source_arg = ("indexSource", HasAttribute("getPortHandleForRouting"))
         a_channel_arg = ("aChannel", int)
         b_channel_arg = ("bChannel", int)
         index_channel_arg = ("indexChannel", int)
@@ -156,50 +163,39 @@ class Encoder(SensorBase):
                 raise ValueError("didn't specify A channel")
             aSource = DigitalInput(aChannel)
             self.allocatedA = True
+            self.addChild(aSource)
         if bSource is None:
             if bChannel is None:
                 raise ValueError("didn't specify B channel")
             bSource = DigitalInput(bChannel)
             self.allocatedB = True
+            self.addChild(bSource)
         if indexSource is None and indexChannel is not None:
             indexSource = DigitalInput(indexChannel)
             self.allocatedIndex = True
+            self.addChild(indexSource)
 
         # save to instance variables
         self.aSource = aSource
         self.bSource = bSource
         self.indexSource = indexSource
         self.encodingType = encodingType
-        self.distancePerPulse = 1.0 # distance of travel for each encoder tick
-        self.pidSource = PIDSource.PIDSourceParameter.kDistance
+        self.pidSource = self.PIDSourceType.kDisplacement
         self._encoder = None
         self.counter = None
-        self.index = 0
-
-        if encodingType == self.EncodingType.k4X:
-            self._encoder, self.index = hal.initializeEncoder(
-                    aSource.getModuleForRouting(),
-                    aSource.getChannelForRouting(),
-                    aSource.getAnalogTriggerForRouting(),
-                    bSource.getModuleForRouting(),
-                    bSource.getChannelForRouting(),
-                    bSource.getAnalogTriggerForRouting(),
-                    reverseDirection)
-            self._encoder_finalizer = \
-                    weakref.finalize(self, _freeEncoder, self._encoder)
-            self.setMaxPeriod(.5)
-            self.encodingScale = 4
-        elif encodingType in (self.EncodingType.k2X, self.EncodingType.k1X):
-            # Use Counter object for 1x and 2x encoding
-            self.counter = Counter(encodingType, aSource, bSource,
-                                   reverseDirection)
-            if encodingType == self.encodingType.k2X:
-                self.encodingScale = 2
-            else:
-                self.encodingScale = 1
-            self.index = self.counter.getFPGAIndex()
-        else:
-            raise ValueError("unrecognized encodingType: %s" % encodingType)
+        self.speedEntry = None
+        self.distanceEntry = None
+        self.distancePerTickEntry = None
+        
+        self._encoder = hal.initializeEncoder(
+            aSource.getPortHandleForRouting(),
+            aSource.getAnalogTriggerTypeForRouting(),
+            bSource.getPortHandleForRouting(),
+            bSource.getAnalogTriggerTypeForRouting(),
+            reverseDirection, encodingType
+        )
+        
+        self.__finalizer = weakref.finalize(self, _freeEncoder, self._encoder)
         
         # Need this to free on unit test wpilib reset
         Resource._add_global_resource(self)
@@ -207,31 +203,37 @@ class Encoder(SensorBase):
         if self.indexSource is not None:
             self.setIndexSource(self.indexSource)
 
-        hal.HALReport(hal.HALUsageReporting.kResourceType_Encoder,
+        self.index = self.getFPGAIndex()
+        hal.report(hal.UsageReporting.kResourceType_Encoder,
                       self.index, encodingType)
-        LiveWindow.addSensorChannel("Encoder", aSource.getChannelForRouting(),
-                                    self)
+        self.setName("Encoder", self.index)
 
     @property
     def encoder(self):
-        if not self._encoder_finalizer.alive:
-            return None
+        if not self.__finalizer.alive:
+            raise ValueError("Cannot use encoder after free() has been called")
         return self._encoder
 
     def getFPGAIndex(self):
         """
+        Get the FPGA Index of the encoder
+
         :returns: The Encoder's FPGA index
+        :rtype: int
         """
-        return self.index
+        return hal.getEncoderFPGAIndex(self.encoder)
 
     def getEncodingScale(self):
         """
         :returns: The encoding scale factor 1x, 2x, or 4x, per the requested
             encodingType. Used to divide raw edge counts down to spec'd counts.
+        :rtype: int
         """
-        return self.encodingScale
+        return hal.getEncoderEncodingScale(self.encoder)
 
     def free(self):
+        """Free the resources used by this object."""
+        super().free()
         if self.aSource is not None and self.allocatedA:
             self.aSource.free()
             self.allocatedA = False
@@ -244,21 +246,17 @@ class Encoder(SensorBase):
         self.aSource = None
         self.bSource = None
         self.indexSource = None
-        if self.counter is not None:
-            self.counter.free()
-            self.counter = None
-        else:
-            self._encoder_finalizer()
+        self.__finalizer()
+        self._encoder = None
 
     def getRaw(self):
         """Gets the raw value from the encoder. The raw value is the actual
         count unscaled by the 1x, 2x, or 4x scale factor.
 
         :returns: Current raw count from the encoder
+        :rtype: int
         """
-        if self.counter is not None:
-            return self.counter.get()
-        return hal.getEncoder(self.encoder)
+        return hal.getEncoderRaw(self.encoder)
 
     def get(self):
         """Gets the current count. Returns the current count on the Encoder.
@@ -266,19 +264,15 @@ class Encoder(SensorBase):
 
         :returns: Current count from the Encoder adjusted for the 1x, 2x, or
             4x scale factor.
+        :rtype: int
         """
-        return int(self.getRaw() * self.decodingScaleFactor())
+        return hal.getEncoder(self.encoder)
 
     def reset(self):
         """Reset the Encoder distance to zero. Resets the current count to
         zero on the encoder.
         """
-        if self.counter is not None:
-            self.counter.reset()
-        elif self.encoder is not None:
-            hal.resetEncoder(self.encoder)
-        else:
-            raise ValueError("operation on freed port")
+        hal.resetEncoder(self.encoder)
 
     def getPeriod(self):
         """Returns the period of the most recent pulse. Returns the period of
@@ -291,15 +285,10 @@ class Encoder(SensorBase):
             :func:`getDistancePerPulse`.
 
         :returns: Period in seconds of the most recent pulse.
+        :rtype: float
         """
         warnings.warn("use getRate instead", DeprecationWarning)
-        if self.counter is not None:
-            measuredPeriod = self.counter.getPeriod() / self.decodingScaleFactor()
-        elif self.encoder is not None:
-            measuredPeriod = hal.getEncoderPeriod(self.encoder)
-        else:
-            raise ValueError("operation on freed port")
-        return measuredPeriod
+        return hal.getEncoderPeriod(self.encoder)
 
     def setMaxPeriod(self, maxPeriod):
         """Sets the maximum period for stopped detection. Sets the value that
@@ -311,13 +300,9 @@ class Encoder(SensorBase):
         :param maxPeriod: The maximum time between rising and falling edges
             before the FPGA will report the device stopped. This is expressed
             in seconds.
+        :type maxPeriod: float
         """
-        if self.counter is not None:
-            self.counter.setMaxPeriod(maxPeriod * self.decodingScaleFactor())
-        elif self.encoder is not None:
-            hal.setEncoderMaxPeriod(self.encoder, maxPeriod)
-        else:
-            raise ValueError("operation on freed port")
+        hal.setEncoderMaxPeriod(self.encoder, maxPeriod)
 
     def getStopped(self):
         """Determine if the encoder is stopped. Using the MaxPeriod value, a
@@ -327,50 +312,33 @@ class Encoder(SensorBase):
 
         :returns: True if the encoder is considered stopped.
         """
-        if self.counter is not None:
-            return self.counter.getStopped()
-        elif self.encoder is not None:
-            return hal.getEncoderStopped(self.encoder)
-        else:
-            raise ValueError("operation on freed port")
+        return hal.getEncoderStopped(self.encoder)
 
     def getDirection(self):
         """The last direction the encoder value changed.
 
         :returns: The last direction the encoder value changed.
+        :rtype: bool
         """
-        if self.counter is not None:
-            return self.counter.getDirection()
         return hal.getEncoderDirection(self.encoder)
 
-    def decodingScaleFactor(self):
-        """The scale needed to convert a raw counter value into a number of
-        encoder pulses.
-        """
-        if self.encodingType == self.EncodingType.k1X:
-            return 1.0
-        elif self.encodingType == self.EncodingType.k2X:
-            return 0.5
-        elif self.encodingType == self.EncodingType.k4X:
-            return 0.25
-        else:
-            raise ValueError("unexpected encodingType: %d" % self.encodingType)
-
     def getDistance(self):
         """Get the distance the robot has driven since the last reset.
 
         :returns: The distance driven since the last reset as scaled by the
             value from :func:`setDistancePerPulse`.
+        :rtype: float
         """
-        return self.getRaw() * self.decodingScaleFactor() * self.distancePerPulse
+        return hal.getEncoderDistance(self.encoder)
 
     def getRate(self):
         """Get the current rate of the encoder. Units are distance per second
         as scaled by the value from :func:`setDistancePerPulse`.
 
-         :returns: The current rate of the encoder.
+        :returns: The current rate of the encoder.
+        :rtype: float
         """
-        return self.distancePerPulse / self.getPeriod()
+        return hal.getEncoderRate(self.encoder)
 
     def setMinRate(self, minRate):
         """Set the minimum rate of the device before the hardware reports it
@@ -378,8 +346,9 @@ class Encoder(SensorBase):
 
         :param minRate: The minimum rate. The units are in distance per
             second as scaled by the value from :func:`setDistancePerPulse`.
+        :type minRate: float
         """
-        self.setMaxPeriod(self.distancePerPulse / minRate)
+        hal.setEncoderMinRate(self.encoder, minRate)
 
     def setDistancePerPulse(self, distancePerPulse):
         """Set the distance per pulse for this encoder. This sets the
@@ -392,8 +361,18 @@ class Encoder(SensorBase):
 
         :param distancePerPulse: The scale factor that will be used to convert
             pulses to useful units.
+        :type distancePerPulse: float
         """
-        self.distancePerPulse = distancePerPulse
+        hal.setEncoderDistancePerPulse(self.encoder, distancePerPulse)
+
+    def getDistancePerPulse(self): 
+        """ 
+        Get the distance per pulse for this encoder.
+
+        :returns: The scale factor that will be used to convert pulses to useful units.
+        :rtype: float
+        """
+        return hal.getEncoderDistancePerPulse(self.encoder)
 
     def setReverseDirection(self, reverseDirection):
         """Set the direction sensing for this encoder. This sets the direction
@@ -402,11 +381,9 @@ class Encoder(SensorBase):
 
         :param reverseDirection: True if the encoder direction should be
             reversed
+        :type reverseDirection: bool
         """
-        if self.counter is not None:
-            self.counter.setReverseDirection(reverseDirection)
-        else:
-            raise NotImplementedError # FIXME?
+        hal.setEncoderReverseDirection(self.encoder, reverseDirection)
 
     def setSamplesToAverage(self, samplesToAverage):
         """Set the Samples to Average which specifies the number of samples
@@ -419,12 +396,9 @@ class Encoder(SensorBase):
 
         :param samplesToAverage: The number of samples to average from 1 to
             127.
+        :type samplesToAverage: int
         """
-        if self.encodingType == self.EncodingType.k4X:
-            hal.setEncoderSamplesToAverage(self.encoder, samplesToAverage)
-        elif self.encodingType in (self.EncodingType.k2X,
-                                   self.EncodingType.k1X):
-            self.counter.setSamplesToAverage(samplesToAverage)
+        hal.setEncoderSamplesToAverage(self.encoder, samplesToAverage)
 
     def getSamplesToAverage(self):
         """Get the Samples to Average which specifies the number of samples
@@ -433,34 +407,33 @@ class Encoder(SensorBase):
         resolution.
 
         :returns: The number of samples being averaged (from 1 to 127)
+        :rtype: int
         """
-        if self.encodingType.value == self.EncodingType.k4X:
-            return hal.getEncoderSamplesToAverage(self.encoder)
-        elif self.encodingType in (self.EncodingType.k2X,
-                                   self.EncodingType.k1X):
-            return self.counter.getSamplesToAverage()
-        else:
-            return 1
+        return hal.getEncoderSamplesToAverage(self.encoder)
 
-    def setPIDSourceParameter(self, pidSource):
+    def setPIDSourceType(self, pidSource):
         """Set which parameter of the encoder you are using as a process
         control variable. The encoder class supports the rate and distance
         parameters.
 
         :param pidSource: An enum to select the parameter.
         """
-        if pidSource not in (0, 1):
-            raise ValueError("invalid pidSource: %s" % pidSource)
+        if pidSource not in (self.PIDSourceType.kDisplacement,
+                             self.PIDSourceType.kRate):
+            raise ValueError("Must be kRate or kDisplacement")
         self.pidSource = pidSource
+        
+    def getPIDSourceType(self):
+        return self.pidSource
 
     def pidGet(self):
         """Implement the PIDSource interface.
 
          :returns: The current value of the selected source parameter.
         """
-        if self.pidSource == self.PIDSourceParameter.kDistance:
+        if self.pidSource == self.PIDSourceType.kDisplacement:
             return self.getDistance()
-        elif self.pidSource == self.PIDSourceParameter.kRate:
+        elif self.pidSource == self.PIDSourceType.kRate:
             return self.getRate()
         else:
             return 0.0
@@ -470,37 +443,24 @@ class Encoder(SensorBase):
         Set the index source for the encoder. When this source rises, the encoder count automatically resets.
 
         :param source: Either an initialized DigitalSource or a DIO channel number
-        :type: Either a :class:`wpilib.DigitalInput` or number
+        :type source: :class:`.DigitalInput` or int
         :param indexing_type: The state that will cause the encoder to reset
-        :type: A value from :class:`wpilib.DigitalInput.IndexingType`
+        :type indexing_type: :class:`.IndexingType`
         """
-        if hasattr(source, "getChannelForRouting"):
+        if hasattr(source, "getPortHandleForRouting"):
             self.indexSource = source
         else:
             self.indexSource = DigitalInput(source)
 
-        activeHigh = (indexing_type == self.IndexingType.kResetWhileHigh or indexing_type == self.IndexingType.kResetOnRisingEdge)
-        edgeSensitive = (indexing_type == self.IndexingType.kResetOnFallingEdge or indexing_type == self.IndexingType.kResetOnRisingEdge)
-        
-        hal.setEncoderIndexSource(self.encoder, self.indexSource.getChannelForRouting(),
-                                  self.indexSource.getAnalogTriggerForRouting(), activeHigh, edgeSensitive)
+        hal.setEncoderIndexSource(self.encoder, self.indexSource.getPortHandleForRouting(),
+                                  self.indexSource.getAnalogTriggerTypeForRouting(), indexing_type)
 
-    # Live Window code, only does anything if live window is activated.
-
-    def getSmartDashboardType(self):
+    def initSendable(self, builder):
         if self.encodingType == self.EncodingType.k4X:
-            return "Quadrature Encoder"
-        return "Encoder"
+            builder.setSmartDashboardType("Quadrature Encoder")
+        else:
+            builder.setSmartDashboardType("Encoder")
 
-    def updateTable(self):
-        table = self.getTable()
-        if table is not None:
-            table.putNumber("Speed", self.getRate())
-            table.putNumber("Distance", self.getDistance())
-            table.putNumber("Distance per Tick", self.distancePerPulse)
-
-    def startLiveWindowMode(self):
-        pass
-
-    def stopLiveWindowMode(self):
-        pass
+        builder.addDoubleProperty("Speed", self.getRate, None)
+        builder.addDoubleProperty("Distance", self.getDistance, None)
+        builder.addDoubleProperty("Distance per Tick", self.getDistancePerPulse, None)
