@@ -1,9 +1,10 @@
-#----------------------------------------------------------------------------
-# Copyright (c) FIRST 2008-2012. All Rights Reserved.
+# validated: 2018-01-19 DS 54a0a7654a91 edu/wpi/first/wpilibj/RobotDrive.java
+# ----------------------------------------------------------------------------
+# Copyright (c) FIRST 2008-2017. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
 # must be accompanied by the FIRST BSD license file in the root directory of
 # the project.
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 import hal
 import math
@@ -12,7 +13,6 @@ import weakref
 
 from .motorsafety import MotorSafety
 from .talon import Talon
-from .canjaguar import CANJaguar
 
 __all__ = ["RobotDrive"]
 
@@ -27,9 +27,13 @@ def _freeRobotDrive(allocatedSpeedControllers):
 
 
 class RobotDrive(MotorSafety):
-    """Operations on a robot drivetrain based on a definition of the motor
+    """
+    .. deprecated:: 2018.0.0
+        Use :class:`.DifferentialDrive` or :class:`.MecanumDrive` instead.
+
+    Operations on a robot drivetrain based on a definition of the motor
     configuration. 
-    
+
     The robot drive class handles basic driving for a robot. Currently, 2
     and 4 motor tank and mecanum drive trains are supported. In the future
     other drive types like swerve might be implemented. Motor channel numbers
@@ -37,20 +41,22 @@ class RobotDrive(MotorSafety):
     the drive function (intended for hand created drive code, such as
     autonomous) or with the Tank/Arcade functions intended to be used for
     Operator Control driving.
+    
+    .. not_implemented: setupMotorSafety
     """
 
     class MotorType:
         """The location of a motor on the robot for the purpose of driving."""
-        
+
         #: Front left
         kFrontLeft = 0
-        
+
         #: Front right
         kFrontRight = 1
-        
+
         #: Rear left
         kRearLeft = 2
-        
+
         #: Rear right
         kRearRight = 3
 
@@ -84,6 +90,9 @@ class RobotDrive(MotorSafety):
         numbers are passed, the motorController keyword argument, if present,
         is the motor controller class to use; if unspecified, :class:`.Talon` is used.
         """
+        warnings.warn(
+            "RobotDrive was deprecated in 2018.0.0. Use DifferentialDrive, KilloughDrive, or MecanumDrive instead.",
+            DeprecationWarning, stacklevel=2)
         super().__init__()
 
         # keyword arguments
@@ -121,60 +130,64 @@ class RobotDrive(MotorSafety):
 
         # convert channel number into motor controller if needed
         if (self.frontLeftMotor is not None and
-            not hasattr(self.frontLeftMotor, "set")):
+                not hasattr(self.frontLeftMotor, "set")):
             self.frontLeftMotor = controllerClass(self.frontLeftMotor)
             self.allocatedSpeedControllers.append(self.frontLeftMotor)
         if (self.rearLeftMotor is not None and
-            not hasattr(self.rearLeftMotor, "set")):
+                not hasattr(self.rearLeftMotor, "set")):
             self.rearLeftMotor = controllerClass(self.rearLeftMotor)
             self.allocatedSpeedControllers.append(self.rearLeftMotor)
         if (self.frontRightMotor is not None and
-            not hasattr(self.frontRightMotor, "set")):
+                not hasattr(self.frontRightMotor, "set")):
             self.frontRightMotor = controllerClass(self.frontRightMotor)
             self.allocatedSpeedControllers.append(self.frontRightMotor)
         if (self.rearRightMotor is not None and
-            not hasattr(self.rearRightMotor, "set")):
+                not hasattr(self.rearRightMotor, "set")):
             self.rearRightMotor = controllerClass(self.rearRightMotor)
             self.allocatedSpeedControllers.append(self.rearRightMotor)
 
-        # all motors start non-inverted
-        self.invertedMotors = [1]*self.kMaxNumberOfMotors
-
         # other defaults
         self.maxOutput = RobotDrive.kDefaultMaxOutput
         self.sensitivity = RobotDrive.kDefaultSensitivity
 
-        self.syncGroup = 0x0
-
         # set up motor safety
         self.setExpiration(self.kDefaultExpirationTime)
         self.setSafetyEnabled(True)
 
-        #Setup Finalizer
-        self._RobotDrive_finalizer = weakref.finalize(self, _freeRobotDrive, self.allocatedSpeedControllers)
+        # Setup Finalizer
+        self.__finalizer = weakref.finalize(self, _freeRobotDrive, self.allocatedSpeedControllers)
 
         # start off not moving
         self.drive(0, 0)
 
-
     def drive(self, outputMagnitude, curve):
-        """Drive the motors at "speed" and "curve".
+        """Drive the motors at "outputMagnitude" and "curve".
 
-        The speed and curve are -1.0 to +1.0 values where 0.0 represents
-        stopped and not turning. The algorithm for adding in the direction
-        attempts to provide a constant turn radius for differing speeds.
+        Both outputMagnitude and curve are -1.0 to +1.0 values, where 0.0
+        represents stopped and not turning. ``curve < 0`` will turn left and ``curve > 0``
+        will turn right.
+
+        The algorithm for steering provides a constant turn radius for any normal
+        speed range, both forward and backward. Increasing m_sensitivity causes
+        sharper turns for fixed values of curve.
 
         This function will most likely be used in an autonomous routine.
 
-        :param outputMagnitude: The forward component of the output magnitude
-            to send to the motors.
-        :param curve: The rate of turn, constant for different forward speeds.
+        :param outputMagnitude: The speed setting for the outside wheel in a turn,
+               forward or backwards, +1 to -1.
+        :param curve: The rate of turn, constant for different forward speeds. Set
+               ``curve < 0`` for left turn or ``curve > 0`` for right turn.
+
+        Set ``curve = e^(-r/w)`` to get a turn radius r for wheelbase w of your robot.
+        Conversely, turn radius r = -ln(curve)*w for a given value of curve and
+        wheelbase w.
         """
         if not RobotDrive.kArcadeRatioCurve_Reported:
-            hal.HALReport(hal.HALUsageReporting.kResourceType_RobotDrive,
-                          self.getNumMotors(),
-                          hal.HALUsageReporting.kRobotDrive_ArcadeRatioCurve)
+            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
+                       self.getNumMotors(),
+                       hal.UsageReporting.kRobotDrive_ArcadeRatioCurve)
             RobotDrive.kArcadeRatioCurve_Reported = True
+
         if curve < 0:
             value = math.log(-curve)
             ratio = (value - self.sensitivity) / (value + self.sensitivity)
@@ -192,6 +205,7 @@ class RobotDrive(MotorSafety):
         else:
             leftOutput = outputMagnitude
             rightOutput = outputMagnitude
+
         self.setLeftRightMotorOutputs(leftOutput, rightOutput)
 
     def tankDrive(self, *args, **kwargs):
@@ -200,7 +214,7 @@ class RobotDrive(MotorSafety):
         Either two joysticks (with optional specified axis) or two raw values
         may be passed positionally, along with an optional squaredInputs
         boolean.  The valid positional combinations are:
-        
+
         - leftStick, rightStick
         - leftStick, rightStick, squaredInputs
         - leftStick, leftAxis, rightStick, rightAxis
@@ -226,6 +240,12 @@ class RobotDrive(MotorSafety):
         :param squaredInputs: Setting this parameter to True decreases the
             sensitivity at lower speeds.  Defaults to True if unspecified.
         """
+        if not RobotDrive.kTank_Reported:
+            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
+                       self.getNumMotors(),
+                       hal.UsageReporting.kRobotDrive_Tank)
+            RobotDrive.kTank_Reported = True
+
         # keyword arguments
         leftStick = kwargs.pop("leftStick", None)
         rightStick = kwargs.pop("rightStick", None)
@@ -276,26 +296,14 @@ class RobotDrive(MotorSafety):
         if squaredInputs is None:
             squaredInputs = True
 
-        # usage reporting
-        if not RobotDrive.kTank_Reported:
-            hal.HALReport(hal.HALUsageReporting.kResourceType_RobotDrive,
-                          self.getNumMotors(),
-                          hal.HALUsageReporting.kRobotDrive_Tank)
-            RobotDrive.kTank_Reported = True
-
         # square the inputs (while preserving the sign) to increase fine
         # control while permitting full power
         leftValue = RobotDrive.limit(leftValue)
         rightValue = RobotDrive.limit(rightValue)
         if squaredInputs:
-            if leftValue >= 0.0:
-                leftValue = (leftValue * leftValue)
-            else:
-                leftValue = -(leftValue * leftValue)
-            if rightValue >= 0.0:
-                rightValue = (rightValue * rightValue)
-            else:
-                rightValue = -(rightValue * rightValue)
+            leftValue = math.copysign(leftValue * leftValue, leftValue)
+            rightValue = math.copysign(rightValue * rightValue, rightValue)
+
         self.setLeftRightMotorOutputs(leftValue, rightValue)
 
     def arcadeDrive(self, *args, **kwargs):
@@ -304,7 +312,7 @@ class RobotDrive(MotorSafety):
         Either one or two joysticks (with optional specified axis) or two raw
         values may be passed positionally, along with an optional
         squaredInputs boolean.  The valid positional combinations are:
-        
+
         - stick
         - stick, squaredInputs
         - moveStick, moveAxis, rotateStick, rotateAxis
@@ -335,6 +343,13 @@ class RobotDrive(MotorSafety):
         :param squaredInputs: Setting this parameter to True decreases the
             sensitivity at lower speeds.  Defaults to True if unspecified.
         """
+
+        if not RobotDrive.kArcadeStandard_Reported:
+            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
+                       self.getNumMotors(),
+                       hal.UsageReporting.kRobotDrive_ArcadeStandard)
+            RobotDrive.kArcadeStandard_Reported = True
+
         # keyword arguments
         stick = kwargs.pop("stick", None)
         moveStick = kwargs.pop("moveStick", None)
@@ -384,26 +399,14 @@ class RobotDrive(MotorSafety):
             squaredInputs = True
 
         # local variables to hold the computed PWM values for the motors
-        if not RobotDrive.kArcadeStandard_Reported:
-            hal.HALReport(hal.HALUsageReporting.kResourceType_RobotDrive,
-                          self.getNumMotors(),
-                          hal.HALUsageReporting.kRobotDrive_ArcadeStandard)
-            RobotDrive.kArcadeStandard_Reported = True
-
         moveValue = RobotDrive.limit(moveValue)
         rotateValue = RobotDrive.limit(rotateValue)
 
         if squaredInputs:
             # square the inputs (while preserving the sign) to increase fine
             # control while permitting full power
-            if moveValue >= 0.0:
-                moveValue = (moveValue * moveValue)
-            else:
-                moveValue = -(moveValue * moveValue)
-            if rotateValue >= 0.0:
-                rotateValue = (rotateValue * rotateValue)
-            else:
-                rotateValue = -(rotateValue * rotateValue)
+            moveValue = math.copysign(moveValue * moveValue, moveValue)
+            rotateValue = math.copysign(rotateValue * rotateValue, rotateValue)
 
         if moveValue > 0.0:
             if rotateValue > 0.0:
@@ -443,18 +446,19 @@ class RobotDrive(MotorSafety):
             to implement field-oriented controls.
         """
         if not RobotDrive.kMecanumCartesian_Reported:
-            hal.HALReport(hal.HALUsageReporting.kResourceType_RobotDrive,
-                          self.getNumMotors(),
-                          hal.HALUsageReporting.kRobotDrive_MecanumCartesian)
+            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
+                       self.getNumMotors(),
+                       hal.UsageReporting.kRobotDrive_MecanumCartesian)
             RobotDrive.kMecanumCartesian_Reported = True
+
         xIn = x
         yIn = y
         # Negate y for the joystick.
         yIn = -yIn
-        # Compenstate for gyro angle.
+        # Compensate for gyro angle.
         xIn, yIn = RobotDrive.rotateVector(xIn, yIn, gyroAngle)
 
-        wheelSpeeds = [0]*self.kMaxNumberOfMotors
+        wheelSpeeds = [0] * self.kMaxNumberOfMotors
         wheelSpeeds[self.MotorType.kFrontLeft] = xIn + yIn + rotation
         wheelSpeeds[self.MotorType.kFrontRight] = -xIn + yIn - rotation
         wheelSpeeds[self.MotorType.kRearLeft] = -xIn + yIn + rotation
@@ -462,14 +466,11 @@ class RobotDrive(MotorSafety):
 
         RobotDrive.normalize(wheelSpeeds)
 
+        self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.maxOutput)
+        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput)
+        self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.maxOutput)
+        self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.maxOutput)
 
-        self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.invertedMotors[self.MotorType.kFrontLeft] * self.maxOutput, self.syncGroup)
-        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.invertedMotors[self.MotorType.kFrontRight] * self.maxOutput, self.syncGroup)
-        self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.invertedMotors[self.MotorType.kRearLeft] * self.maxOutput, self.syncGroup)
-        self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.invertedMotors[self.MotorType.kRearRight] * self.maxOutput, self.syncGroup)
-
-        if self.syncGroup != 0:
-            CANJaguar.updateSyncGroup(self.syncGroup)
         self.feed()
 
     def mecanumDrive_Polar(self, magnitude, direction, rotation):
@@ -488,10 +489,11 @@ class RobotDrive(MotorSafety):
             independent of the magnitute or direction. [-1.0..1.0]
         """
         if not RobotDrive.kMecanumPolar_Reported:
-            hal.HALReport(hal.HALUsageReporting.kResourceType_RobotDrive,
-                          self.getNumMotors(),
-                          hal.HALUsageReporting.kRobotDrive_MecanumPolar)
+            hal.report(hal.UsageReporting.kResourceType_RobotDrive,
+                       self.getNumMotors(),
+                       hal.UsageReporting.kRobotDrive_MecanumPolar)
             RobotDrive.kMecanumPolar_Reported = True
+
         # Normalized for full power along the Cartesian axes.
         magnitude = RobotDrive.limit(magnitude) * math.sqrt(2.0)
         # The rollers are at 45 degree angles.
@@ -499,7 +501,7 @@ class RobotDrive(MotorSafety):
         cosD = math.cos(dirInRad)
         sinD = math.sin(dirInRad)
 
-        wheelSpeeds = [0]*self.kMaxNumberOfMotors
+        wheelSpeeds = [0] * self.kMaxNumberOfMotors
         wheelSpeeds[self.MotorType.kFrontLeft] = sinD * magnitude + rotation
         wheelSpeeds[self.MotorType.kFrontRight] = cosD * magnitude - rotation
         wheelSpeeds[self.MotorType.kRearLeft] = cosD * magnitude + rotation
@@ -507,14 +509,11 @@ class RobotDrive(MotorSafety):
 
         RobotDrive.normalize(wheelSpeeds)
 
+        self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.maxOutput)
+        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.maxOutput)
+        self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.maxOutput)
+        self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.maxOutput)
 
-        self.frontLeftMotor.set(wheelSpeeds[self.MotorType.kFrontLeft] * self.invertedMotors[self.MotorType.kFrontLeft] * self.maxOutput, self.syncGroup)
-        self.frontRightMotor.set(wheelSpeeds[self.MotorType.kFrontRight] * self.invertedMotors[self.MotorType.kFrontRight] * self.maxOutput, self.syncGroup)
-        self.rearLeftMotor.set(wheelSpeeds[self.MotorType.kRearLeft] * self.invertedMotors[self.MotorType.kRearLeft] * self.maxOutput, self.syncGroup)
-        self.rearRightMotor.set(wheelSpeeds[self.MotorType.kRearRight] * self.invertedMotors[self.MotorType.kRearRight] * self.maxOutput, self.syncGroup)
-
-        if self.syncGroup != 0:
-            CANJaguar.updateSyncGroup(self.syncGroup)
         self.feed()
 
     def holonomicDrive(self, magnitude, direction, rotation):
@@ -549,25 +548,23 @@ class RobotDrive(MotorSafety):
         rightOutput = RobotDrive.limit(rightOutput) * self.maxOutput
 
         if self.frontLeftMotor is not None:
-            self.frontLeftMotor.set(leftOutput * self.invertedMotors[self.MotorType.kFrontLeft], self.syncGroup)
-        self.rearLeftMotor.set(leftOutput * self.invertedMotors[self.MotorType.kRearLeft], self.syncGroup)
+            self.frontLeftMotor.set(leftOutput)
+        self.rearLeftMotor.set(leftOutput)
 
         if self.frontRightMotor is not None:
-            self.frontRightMotor.set(-rightOutput * self.invertedMotors[self.MotorType.kFrontRight], self.syncGroup)
-        self.rearRightMotor.set(-rightOutput * self.invertedMotors[self.MotorType.kRearRight], self.syncGroup)
+            self.frontRightMotor.set(-rightOutput)
+        self.rearRightMotor.set(-rightOutput)
 
-        if self.syncGroup != 0:
-            CANJaguar.updateSyncGroup(self.syncGroup)
         self.feed()
 
     @staticmethod
-    def limit(num):
+    def limit(number):
         """Limit motor values to the -1.0 to +1.0 range."""
-        if num > 1.0:
+        if number > 1.0:
             return 1.0
-        if num < -1.0:
+        if number < -1.0:
             return -1.0
-        return num
+        return number
 
     @staticmethod
     def normalize(wheelSpeeds):
@@ -582,8 +579,9 @@ class RobotDrive(MotorSafety):
     @staticmethod
     def rotateVector(x, y, angle):
         """Rotate a vector in Cartesian space."""
-        cosA = math.cos(math.radians(angle))
-        sinA = math.sin(math.radians(angle))
+        angle = math.radians(angle)
+        cosA = math.cos(angle)
+        sinA = math.sin(angle)
         return (x * cosA - y * sinA), (x * sinA + y * cosA)
 
     def setInvertedMotor(self, motor, isInverted):
@@ -597,7 +595,16 @@ class RobotDrive(MotorSafety):
         :param motor: The motor index to invert.
         :param isInverted: True if the motor should be inverted when operated.
         """
-        self.invertedMotors[motor] = -1 if isInverted else 1
+        if motor == self.MotorType.kFrontLeft:
+            self.frontLeftMotor.setInverted(isInverted)
+        elif motor == self.MotorType.kFrontRight:
+            self.frontRightMotor.setInverted(isInverted)
+        elif motor == self.MotorType.kRearLeft:
+            self.rearLeftMotor.setInverted(isInverted)
+        elif motor == self.MotorType.kRearRight:
+            self.rearRightMotor.setInverted(isInverted)
+        else:
+            raise ValueError("Invalid motor type specified")
 
     def setSensitivity(self, sensitivity):
         """Set the turning sensitivity.
@@ -618,19 +625,27 @@ class RobotDrive(MotorSafety):
         """
         self.maxOutput = maxOutput
 
+    def free(self):
+        self.__finalizer()
+        self.frontLeftMotor = None
+        self.frontRightMotor = None
+        self.rearLeftMotor = None
+        self.rearRightMotor = None
+        self.allocatedSpeedControllers = list()
+        self.setSafetyEnabled(False)
 
     def getDescription(self):
         return "Robot Drive"
 
     def stopMotor(self):
         if self.frontLeftMotor is not None:
-            self.frontLeftMotor.set(0.0)
+            self.frontLeftMotor.stopMotor()
         if self.frontRightMotor is not None:
-            self.frontRightMotor.set(0.0)
+            self.frontRightMotor.stopMotor()
         if self.rearLeftMotor is not None:
-            self.rearLeftMotor.set(0.0)
+            self.rearLeftMotor.stopMotor()
         if self.rearRightMotor is not None:
-            self.rearRightMotor.set(0.0)
+            self.rearRightMotor.stopMotor()
         self.feed()
 
     def getNumMotors(self):
@@ -640,22 +655,3 @@ class RobotDrive(MotorSafety):
         if self.rearLeftMotor is not None: motors += 1
         if self.rearRightMotor is not None: motors += 1
         return motors
-
-    def setCANJaguarSyncGroup(self, syncGroup):
-        """
-        Set the number of the sync group for the motor controllers. If the motor controllers are :class:`CANJaguar`s,
-        then they will be added to this sync group, causing them to update their values at the same time.
-
-        :param syncGroup: The update group to add the motor controllers to.
-        """
-        self.syncGroup = syncGroup
-
-    def free(self):
-        self._RobotDrive_finalizer()
-        self.frontLeftMotor = None
-        self.frontRightMotor = None
-        self.rearLeftMotor = None
-        self.rearRightMotor = None
-        self.allocatedSpeedControllers = list()
-        self.setSafetyEnabled(False)
-
