@@ -1,3 +1,4 @@
+# validated: 2017-10-07 EN e1195e8b9dab edu/wpi/first/wpilibj/SafePWM.java
 #----------------------------------------------------------------------------
 # Copyright (c) FIRST 2008-2012. All Rights Reserved.
 # Open Source Software - may be modified and shared by FRC teams. The code
@@ -11,9 +12,8 @@ from .pwm import PWM
 __all__ = ["SafePWM"]
 
 class SafePWM(PWM, MotorSafety):
-    """A raw PWM interface that implements the :class:`.MotorSafety` interface
-    
-        .. not_implemented: initSafePWM
+    """
+        A raw PWM interface that implements the :class:`.MotorSafety` interface
     """
     
     def __init__(self, channel):
@@ -39,4 +39,4 @@ class SafePWM(PWM, MotorSafety):
         return "PWM %d" % self.getChannel()
 
     def disable(self):
-        self.setRaw(self.kPwmDisabled)
+        self.setDisabled()
